# -*- coding: utf-8 -*-

"""
####################################################################
#                                                                  #
#                           AWI Basemap                            #
#                         CREATE BASEMAPS                          #
#                                                                  #
####################################################################

This script creates the final AWI Basemaps in all three projections
and with different color palettes. This happens in two setps. First
the DEM is rendered with the color palette into a RGB color raster.
Then the shaded map is calculated by combining the colored raster
with the shading raster.
At the end, the raster files are finalized by creating tiles and
pyramids and by adding metadata tags.

"""


#=============================================================================
#    SCRIPT INFO
#=============================================================================

__author__ = 'Simon Dreutter'
__version__ = '0.3'
__date__ = '2021-11-05'
__email__ = 'simon.dreutter@awi.de'
__status__ = 'Developement'


#=============================================================================
#    IMPORT
#=============================================================================

import rasterio

# import AWI Basemap configuration
from config import *


#=============================================================================
#    GLOBAL VARIABLES
#=============================================================================

# GDAL creation options for final result
GDAL_CREATE_OPTIONS_FINAL = '-co TILED=YES -co BLOCKXSIZE=512 -co BLOCKYSIZE=512 -co COPY_SRC_OVERVIEWS=FALSE -co COMPRESS=NONE -co BIGTIFF=IF_NEEDED'

# Method to create overviews
OVERVIEW_METHOD = 'cubic'


#=============================================================================
#    FUNCTIONS
#=============================================================================

def create_basemap(name, grid, hillshade, color_palette, epsg, output):
    """Create the AWI Basemap RGB grid from various inputs

    Parameters
    ----------
    name : str
        Name of grid version
    grid : str
        Path to input DEM grid
    hillshade : str
        Path to input hillshade grid
    color_palette : str
        Path to input color palette file
    epsg : str
        EPSG code
    output : str
        Path to output RGB grid

    """
    # create RGB version with grid and color palette
    msg = f'Creating {name} RGB...'
    cmd = (f'gdaldem'
           f' color-relief'
           f' {grid}'
           f' {color_palette}'
           f' {TMP_RGB}'
           f' -of GTiff'
           f' -b 1'
           f' -compute_edges'
           f' {GDAL_CREATE_OPTIONS}')
    run(msg,cmd)
    
    # compile shaded and colored RGB grid
    msg = f'Creating shaded {name} map...'
    cmd = (f'gdal_calc'
           f' --calc "uint8(((A/255.)*(((B*0.75)+(255.*0.25))/255.))*254)"'
           f' --format GTiff'
           f' --type Byte'
           f' -A {TMP_RGB}'
           f' --A_band 1'
           f' -B {hillshade}'
           f' --B_band 1'
           f' --allBands=A'
           f' --overwrite'
           f' --outfile {TMP_SHADED}')
    # {GDAL_CREATE_OPTIONS_PY}
    run(msg,cmd)
    
    # cleanup
    cleanup(TMP_RGB)
    
    # finalize grid with GDAL create options
    msg = f'Finalizing {name}...'
    cmd = (f'gdal_translate'
           f' -a_srs {epsg}'
           f' -stats'
           f' -of GTiff'
           f' {GDAL_CREATE_OPTIONS_FINAL}'
           f' {TMP_SHADED} {output}')
    run(msg,cmd)
    
    # cleanup
    cleanup(TMP_SHADED)
    cleanup(f'{TMP_SHADED}.aux.xml')
    
    # create internal pyramid layers
    msg = f'Creating pyramids for {name}...'
    cmd = (f'gdaladdo'
           f' -r {OVERVIEW_METHOD}'
           f' --config COMPRESS_OVERVIEW NONE'
           f' --config INTERLEAVE_OVERVIEW PIXEL'
           f' --config BIGTIFF_OVERVIEW IF_NEEDED'
           f' --config GDAL_TIFF_OVR_BLOCKSIZE 512'
           f' {output}'
           f' 2 4 8 16 32 64')
    run(msg,cmd)
    
    # set metadata in GeoTIFF tags
    print('====================================================================')
    print()
    print(f'Updating Metadata for {name}...')
    METADATA['NAME'] = name
    METADATA['EPSG'] = epsg
    with rasterio.open(output, 'r+') as ds:
        # update metadata
        ds.update_tags(**METADATA)
    print()


#=============================================================================
#    PROCESS
#=============================================================================

# heading
print()
print('====================================================================')
print(f'{"CREATE BASEMAPS":^68}')

# create the different basemaps with map parameters from config.py
for kwargs in MAPS:
    create_basemap(**kwargs)

# cleanup
cleanup(GEBCO_WORLD)
cleanup(GEBCO_WORLD_HILLSHADE)
cleanup(GEBCO_ARCTIC)
cleanup(GEBCO_ARCTIC_HILLSHADE)
cleanup(GEBCO_ANTARCTIC)
cleanup(GEBCO_ANTARCTIC_HILLSHADE)


#=============================================================================
#    METADATA
#=============================================================================

# write metadata to text file
with open(METADATA_FILE, 'w') as f:
    for key, val in METADATA.items():
        if not key in ['NAME','EPSG']:
            f.write(f'====================================================================\n')
            f.write(f'{key.replace("_"," "):^68}\n')
            f.write(f'====================================================================\n')
            f.write(f'{val}\n\n\n')
